function shift = findLeadingShock(snapshotMatrix, y_z)  

% This function tracks the leading shock wave in snapshotMatrix based on
% the threshold y_z.
%
% Output arguments:
% - shift: vector of dimension m, where m is the number of time steps;
%   contains the location of the leading shock (relative to the center of
%   the computational domain) at each discrete time point.
%
%
% Input arguments:
% - snapshotMatrix: matrix of dimension Nxm, where m is the number of time
%   steps; each column contains a snapshot of e.g. a discretized solution
%   of a partial differential equation
%
% - y_z: scalar; specifies the threshold value used for tracking the shock
%
%
% Authors:
%   Julius Reiss, reiss@tnt.tu-berlin.de
%   Philipp Schulze, pschulze@math.tu-berlin.de
%
% Version:
%   1.0 (February 15, 2018)
% 

[N, nTimeSteps] = size(snapshotMatrix) ;
shift           = zeros(nTimeSteps,1)  ; % initialization of shift

% tracking the location of the shock in each column of snapshotMatrix
for n=1:nTimeSteps
    i_up = find(snapshotMatrix(:,n)>y_z, 1, 'last') ; % find last value in current column of snapshotMatrix which is bigger than y_Z
    y1 = snapshotMatrix(i_up-1,n); % value left from the shock
    y2 = snapshotMatrix(i_up  ,n); % value right from the shock
    alpha = (y_z-y1)/(y2-y1) ; % weighting factor resulting from a linear interpolation between y1 and y2
    if isempty(i_up) % if no value of current snapshot is bigger than y_z, set current shift zero
       shift(n) = 0 ; 
    else     
       shift(n) = i_up-1+alpha ; % set current shift to a value between i_up-1 and i_up based on a linear interpolation between y1 and y2
    end
end

% subtract round(N/2) from shift to describe shift relative to the center
% of the computational domain
shift = shift-round(N/2) ;