function qsTilde = additiveSubspaceCombination(params, sPODsystem, nModes)   

% This function constructs the sPOD approximation based on the sPOD modes.
%
% Output arguments:
% - qsTilde: tensor of same dimension as the snapshot tensor qs; contains
%   the sPOD approximation
%
%
% Input arguments:
% - params: struct containg parameters which specify the case.
%
% - sPODsystem: struct containing the sPOD modes, their amplitudes, the
%   shift matrices, and the number of frames.
%
% - nModes (optional): vector of dimension 1xk, where k is the number of 
%   frames; contains the number of modes per frame which shall be taken 
%   for the sPOD approximation
%
%
% Authors:
%   Julius Reiss, reiss@tnt.tu-berlin.de
%   Philipp Schulze, pschulze@math.tu-berlin.de
%
% Version:
%   1.0 (February 15, 2018)
% 

% derived data from params
Nxi                    = params.geometry.Nxi                                             ; 
nVars                  = params.data.nVars                                               ; 
nTimeSteps             = params.time.nTimeSteps                                          ;
shiftDirection         = giveStructValue(params, 'mode', 'shiftDirection', 1)            ;
negativeShiftsViaRot90 = giveStructValue(params, 'mode', 'negativeShiftsViaRot90', true) ;

f = @(x)rot90(x,2) ; % f is needed in case that negativeShiftsViaRot90 is true

% constructing the sPOD approximation
switch params.geometry.nDimensions
    case 1 % 1D case
        qsTilde = zeros(Nxi, nVars, nTimeSteps) ;
        for i = 1:sPODsystem.nSubSpaces % loop over velocity frames
            if(nargin<3)
                nModes(i) = min(params.mode.nModes(i),size(sPODsystem.U{i},2)) ;
            else
                nModes(i) = min(nModes(i),size(sPODsystem.U{i},2)) ;
            end
            qsTildeSubSpace = reshape(sPODsystem.U{i}(:,1:nModes(i))*sPODsystem.S{i}(1:nModes(i),1:nModes(i))*sPODsystem.V{i}(:,1:nModes(i))', Nxi, nVars, nTimeSteps) ; % component of the ith velocity frame (in co-moving frame)
            if(~negativeShiftsViaRot90)
                qsTildeSubSpaceShiftedBack = shiftMatrix(params, qsTildeSubSpace, 1, sPODsystem.negShiftMatrices{i}) ; % component of the ith velocity frame (in lab frame)
            else
                qsTildeSubSpaceShiftedBack = shiftMatrix(params, qsTildeSubSpace, 1, cellfun(f,sPODsystem.shiftMatrices{i},'UniformOutput',false)) ; % component of the ith velocity frame (in lab frame)
            end
            qsTilde = qsTilde+qsTildeSubSpaceShiftedBack ; 
        end
    case 2 % 2D case
        qsTilde = zeros(Nxi(1), Nxi(2), nVars, nTimeSteps) ;
        for i = 1:sPODsystem.nSubSpaces % loop over velocity frames
            if(nargin<3)
                nModes(i) = min(params.mode.nModes(i),size(sPODsystem.U{i},2)) ;
            else
                nModes(i) = min(nModes(i),size(sPODsystem.U{i},2)) ;
            end
            qsTildeSubSpace = reshape(sPODsystem.U{i}(:,1:nModes(i))*sPODsystem.S{i}(1:nModes(i),1:nModes(i))*sPODsystem.V{i}(:,1:nModes(i))', Nxi(1), Nxi(2), nVars, nTimeSteps) ; % component of the ith velocity frame (in co-moving frame)
            if(~negativeShiftsViaRot90)
                qsTildeSubSpaceShiftedBack = shiftMatrix(params, qsTildeSubSpace, shiftDirection, sPODsystem.negShiftMatrices{i}) ; % component of the ith velocity frame (in lab frame)
            else
                qsTildeSubSpaceShiftedBack = shiftMatrix(params, qsTildeSubSpace, shiftDirection, cellfun(f,sPODsystem.shiftMatrices{i},'UniformOutput',false)) ; % component of the ith velocity frame (in lab frame)   
            end
            qsTilde = qsTilde+qsTildeSubSpaceShiftedBack ; 
        end
    otherwise
        error('The spatial dimension should be between 1 and 2.')
end