function NewSPODSystem = bestFitSPODSystem(params, qs, combinedSPODsystem)

% This function solves the least squares problem in the sPOD algorithm.
%
% Output arguments:
% - NewSPODSystem: struct containing sPOD modes and amplitudes obtained
%   based on the solution of the least squares problem. 
%
%
% Input arguments:
% - params: struct containg parameters which specify the case.
%
% - qs: snapshot tensor
%
% - combinedSPODSystem: struct containing the modes and (normalized) 
%   amplitudes of the current sPOD approximation and of the residual (also 
%   the shift matrices and the number of frames)
%
%
% Authors:
%   Julius Reiss, reiss@tnt.tu-berlin.de
%   Philipp Schulze, pschulze@math.tu-berlin.de
%
% Version:
%   1.0 (February 15, 2018)
% 

% derived data from combinedSPODsystem
U          = combinedSPODsystem.U          ;  
V          = combinedSPODsystem.V          ; 
nSubSpaces = combinedSPODsystem.nSubSpaces ;  

% derived data from params
Nxi                    = params.geometry.Nxi                                             ; 
nVars                  = params.data.nVars                                               ; 
nTimeSteps             = params.time.nTimeSteps                                          ;
nModes                 = params.mode.nModes                                              ;
shiftDirection         = giveStructValue(params, 'mode', 'shiftDirection', 1)            ;
negativeShiftsViaRot90 = giveStructValue(params, 'mode', 'negativeShiftsViaRot90', true) ;
useAllModes            = giveStructValue(params, 'mode', 'useAllModes', true)            ;

% quantities specifying array dimensions in the following
nDiscreteVars  = prod(Nxi)*nVars    ; % length of one snapshot
nCombinedModes = NaN(1, nSubSpaces) ; % initialization of the vector containing the number of available modes for each frame
for i=1:nSubSpaces
    nCombinedModes(i) = size(U{i}, 2) ;
end
nAllSubSpaces  = sum(nCombinedModes)      ; % total number of available modes
nRows 	       = nDiscreteVars*nTimeSteps ; % number of elements in the snapshot tensor

% some initializations
ansatzSpaceCoMoving         = NaN(nRows, nAllSubSpaces) ; % matrix where each column contains a (reshaped) dyadic product of a mode of combinedSPODsystem and its corresponding (normalized) amplitude (i.e. in the co-moving frame)
ansatzSpace                 = NaN(nRows, nAllSubSpaces) ; % same as "ansatzSpaceCoMoving" but in the lab frame
indexOffset                 = zeros(nSubSpaces+1, 1)    ; % auxiliary variable which sums up the entries of nCombinedModes
NewSPODSystem               = combinedSPODsystem        ;
sPODApproximationComponents = cell(nSubSpaces, 1)       ; % components which are used for computing NewSPODSystem and based on the solution of the least squares problem

f = @(x)rot90(x,2) ; % f is needed in case that negativeShiftsViaRot90 is true

% filling the columns of "ansatzSpaceCoMoving" and "ansatzSpace"
switch params.geometry.nDimensions
    case 1 % 1D case
        for i = 1:nSubSpaces % loop over velocity frames
            for j = 1:nCombinedModes(i) % loop over available modes in the current velocity frame  
                ansatzSpaceTemp                         = U{i}(:,j)*V{i}(:,j)'               ; % dyadic product of the current mode and its (normalized) amplitude                   
                ansatzSpaceCoMoving(:,indexOffset(i)+j) = reshape(ansatzSpaceTemp, nRows, 1) ;
                if(~negativeShiftsViaRot90)
                    ansatzSpace(:,indexOffset(i)+j) = reshape(shiftMatrix(params,reshape(ansatzSpaceTemp,Nxi,nVars,nTimeSteps),1,combinedSPODsystem.negShiftMatrices{i}), nRows, 1) ;  
                else
                    ansatzSpace(:,indexOffset(i)+j) = reshape(shiftMatrix(params,reshape(ansatzSpaceTemp,Nxi,nVars,nTimeSteps),1,cellfun(f,combinedSPODsystem.shiftMatrices{i},'UniformOutput',false)), nRows, 1) ;  
                end
            end
            indexOffset(i+1) = indexOffset(i)+nCombinedModes(i) ; 
        end    
    case 2 % 2D case
        for i = 1:nSubSpaces % loop over velocity frames
            for j = 1:nCombinedModes(i) % loop over available modes in the current velocity frame 
                ansatzSpaceTemp                         = U{i}(:,j)*V{i}(:,j)'               ; % dyadic product of the current mode and its (normalized) amplitude                   
                ansatzSpaceCoMoving(:,indexOffset(i)+j) = reshape(ansatzSpaceTemp, nRows, 1) ;  
                if(~negativeShiftsViaRot90)
                    ansatzSpace(:,indexOffset(i)+j) = reshape(shiftMatrix(params,reshape(ansatzSpaceTemp,Nxi(1),Nxi(2),nVars,nTimeSteps),shiftDirection,combinedSPODsystem.negShiftMatrices{i}), nRows, 1) ;  
                else
                    ansatzSpace(:,indexOffset(i)+j) = reshape(shiftMatrix(params,reshape(ansatzSpaceTemp,Nxi(1),Nxi(2),nVars,nTimeSteps),shiftDirection,cellfun(f,combinedSPODsystem.shiftMatrices{i},'UniformOutput',false)), nRows, 1) ;
                end
                   
            end
            indexOffset(i+1) = indexOffset(i)+nCombinedModes(i) ; 
        end 
    otherwise
        error('The spatial dimension should be between 1 and 2.')
end                                   

% solving the least squares problem
switch params.mode.lsqr.solver
    case 'backslash' % via backslash
        alpha = ansatzSpace\reshape(qs,[],1) ; % solution of the least squares problem
    case 'lsqr' % using the solver "lsqr"
        tol        = params.mode.lsqr.tol                              ;
        maxIter    = params.mode.lsqr.maxIter                          ;
        [alpha, ~] = lsqr(ansatzSpace, reshape(qs,[],1), tol, maxIter) ; % solution of the least squares problem
    otherwise
        error('Unknown least squares solver.')
end

clear ansatzSpace ansatzSpaceTemp

% computing NewSPODSystem
for i = 1:nSubSpaces % loop over the velocity frames
    currentIndices                 = indexOffset(i)+1:indexOffset(i+1)                           ; % indices of the current modes
    sPODApproximationComponents{i} = ansatzSpaceCoMoving(:,currentIndices)*alpha(currentIndices) ; % sPODApproximationComponents is obtained by substituting alpha into the least squares ansatz
    if(useAllModes)
        [NewSPODSystem.U{i}, S, NewSPODSystem.V{i}] = svds(reshape(sPODApproximationComponents{i},nDiscreteVars,nTimeSteps), nModes(i)) ; % NewSPODSystem is obtained via a truncated svd of sPODApproximationComponents
    else
        nModesSVD                                   = giveStructValue(params, 'mode', 'nModesSVD', nCombinedModes(i))                                          ;
        [NewSPODSystem.U{i}, S, NewSPODSystem.V{i}] = svds(reshape(sPODApproximationComponents{i},nDiscreteVars,nTimeSteps), min(nCombinedModes(i),nModesSVD)) ; % NewSPODSystem is obtained via a truncated svd of sPODApproximationComponents
    end
    NewSPODSystem.S{i} = sparse(S) ; 
end