function sPODsystem = multiShiftReduce(params, sPODsystem, qs, nModes)  

% This function performs the MS&R procedure.
%
% Output arguments:
% - sPODsystem: struct containing modes and amplitudes of the as well as
%   shift matrices and the number of frames
%
%
% Input arguments:
% - params: struct containg parameters which specify the case.
%
% - sPODsystem: struct containing shift matrices and the number of frames
%
% - qs: snapshot tensor
%
% - nModes: vector of dimension 1xk, where k is the number of frames; 
%   contains the number of modes per frame which shall be taken for MS&R
%
%
% Authors:
%   Julius Reiss, reiss@tnt.tu-berlin.de
%   Philipp Schulze, pschulze@math.tu-berlin.de
%
% Version:
%   1.0 (February 15, 2018)
% 

% derived data from params and sPODsystem
nSubSpaces     = sPODsystem.nSubSpaces                                ; 
nTimeSteps     = params.time.nTimeSteps                               ;
nRows          = prod(params.geometry.Nxi)*params.data.nVars          ; % length of one snapshot
shiftDirection = giveStructValue(params, 'mode', 'shiftDirection', 1) ;

% some initializations
sPODsystem.U = cell(nSubSpaces, 1) ; % modes   
sPODsystem.V = cell(nSubSpaces, 1) ; % (normalized) amplitudes 
sPODsystem.S = cell(nSubSpaces, 1) ; % factors to obtain the "un-normalized" amplitudes

for i = 1:nSubSpaces % loop over the velocity frames
    qsShifted = shiftMatrix(params, qs, shiftDirection, sPODsystem.shiftMatrices{i}) ; % shifted snapshot tensor
    qsShifted = reshape(qsShifted, nRows, nTimeSteps)                                ;
	nModesSVD = giveStructValue(params, 'mode', 'nModesSVD', min(size(qsShifted)))   ;
    [U, S, V] = svds(qsShifted, nModesSVD)                                           ;
    k         = nModes(i)                                                            ;
    while(size(U,2)<nModes(i)) % this while loop enforces that size(U,2) is at least nModes(i)
        k         = k+1                ;
        [U, S, V] = svds(qsShifted, k) ;
    end
    sPODsystem.U{i} = U(:,1:nModes(i))                   ;
    sPODsystem.V{i} = V(:,1:nModes(i))                   ;
    sPODsystem.S{i} = sparse(S(1:nModes(i),1:nModes(i))) ;   
end