function [sPodSystem, qsTilde, relErrors] = subspaceDecompositionBySPODiteration(params, qs, cs)  

% This function performs the sPOD algorithm.
%
% Output arguments:
% - sPodSystem: struct containing sPOD modes, amplitudes, shift matrices, 
%   and the number of frames
%
% - qsTilde: tensor of same dimension as the snapshot tensor qs; contains
%   the sPOD approximation
%
% - relErrors: vector of dimension params.mode.nCleaningIterations 
%   containing the relative mean error for each cleaning iteration
%
%
% Input arguments:
% - params: struct containing parameters which specify the case.
%
% - qs: snapshot tensor
%
% - cs: matrix of dimension mxk, where m is the number of time steps and k
%   is the number of velocity frames; OR a tensor of dimension mxlxk, where
%   l is the number of spatial dimensions; contains the shift values for
%   each velocity frame and each discrete time step (and for each space 
%   dimension in the tensor case)
%
%
% Authors:
%   Julius Reiss, reiss@tnt.tu-berlin.de
%   Philipp Schulze, pschulze@math.tu-berlin.de
%
% Version:
%   1.0 (February 15, 2018)
% 

% derived data from params
errorMeasure             = giveStructValue(params, 'mode', 'errorMeasure', @relErrorVectorNorm) ; % function for computing the relative mean error
minNumberIterations      = giveStructValue(params, 'mode', 'minNumberIterations', 4)            ; % minimum number of cleaning iterations
rangeOfDeclineControl    = giveStructValue(params, 'mode', 'rangeOfDeclineControl', 6)          ; % parameter for determining when the sPOD algorithm is converged and can be aborted
minDeclineFactor         = giveStructValue(params, 'mode', 'minDeclineFactor', 0.01)            ; % parameter for determining when the sPOD algorithm is converged and can be aborted
saveFirstIter            = giveStructValue(params, 'mode', 'saveFirstIter', false)              ;
useConstVelocity         = giveStructValue(params, 'mode', 'useConstVelocity', false)           ;
useAllModes              = giveStructValue(params, 'mode', 'useAllModes', true)                 ;
checkConvergenceCriteria = giveStructValue(params, 'mode', 'checkConvergenceCriteria', false)   ;
nModes                   = params.mode.nModes                                                   ;
nCleaningIterations      = params.mode.nCleaningIterations                                      ;
nErrorModes              = nModes*params.mode.nErrorModesPerMode                                ; % number of modes for MS&R of the residual

% initialize sPodSystem which initially contains the shift matrices and the
% number of frames
if(useConstVelocity)
    [sPodSystem, params] = initShiftMatrix_constVelocity(params, params.SLdetection.velocity) ;  
else
    [sPodSystem, params] = initShiftMatrix(params, cs) ;  
end

% first MS&R of the snapshot matrix
sPodSystem = multiShiftReduce(params, sPodSystem, qs, params.mode.nModes) ; % sPodSystem gets additionally modes and amplitudes

% if saveFirstIter is true, then sPodSystem resulting from the first MS&R
% is stored
if(saveFirstIter)
    if(~exist('RESULTS','dir'))
        mkdir('RESULTS')
    end
    save('RESULTS/firstIter', 'sPodSystem')
end

% sPOD approximation is constructed based on sPodSystem
if(useAllModes)
    qsTilde = additiveSubspaceCombination(params, sPodSystem) ;
else
    qsTilde = additiveSubspaceCombination(params, sPodSystem, min(nModes,1)) ; 
end

% some initializations
relErrors  = zeros(nCleaningIterations+1, 1) ;
diffSystem = sPodSystem                      ; % diffSystem contains the modes and amplitudes of the residual
if(checkConvergenceCriteria)
       orthoError  =  cell(nCleaningIterations, 1) ; % contains the orthogonality errors
       Sdiff       =  cell(nCleaningIterations, 1) ; % contains the singular values of the residual
       Sall        =  cell(nCleaningIterations, 1) ; % contains the singular values of the sPOD approximation
       sPodSystems = cell(nCleaningIterations, 1)  ; % contains sPodSystem after each cleaning iteration
end

disp('MS&R')

for l=1:nCleaningIterations % cleaning iterations
    qsDiff       = qsTilde -qs              ; % residual
    relErrors(l) = errorMeasure(qs, qsDiff) ;  
    disp(['relative error: ',num2str(relErrors(l))])
    if(l>=minNumberIterations) % if minimum number of iterations is reached, check convergence
        if((relErrors(l)>=relErrors(l-1))&&(relErrors(l-1)>=relErrors(l-2))) % if error has increased two times in a row, stop
            relErrors(l+1:nCleaningIterations) = relErrors(l) ;
            break
        end
        if(l>=max(minNumberIterations,rangeOfDeclineControl+1)) % if error decays too slowly, stop
            if(relErrors(l)>=relErrors(l-rangeOfDeclineControl)*(1-minDeclineFactor))
                relErrors(l+1:nCleaningIterations) = relErrors(l) ;
                break
            end
        end
    end
    diffSystem = multiShiftReduce(params, diffSystem, qsDiff, nErrorModes) ; % MS&R of the residual
    if(useAllModes)
        sPodAnsatz = combineModes(sPodSystem, diffSystem) ; % collects quantities contained in sPodSystem and diffSystem and puts them into sPodAnsatz
    else
        sPodAnsatz = combineModes(sPodSystem, diffSystem, min(nModes,l+1), min(nModes,2*(l+1))) ; % collects quantities contained in sPodSystem and diffSystem and puts them into sPodAnsatz
    end
    sPodSystem = bestFitSPODSystem(params, qs, sPodAnsatz)       ; % solves the least squares problem to "clean" the modes
    qsTilde    = additiveSubspaceCombination(params, sPodSystem) ; % constructs sPOD approximation based on cleaned modes
    if(checkConvergenceCriteria)
        [orthoError{l} , Sdiff{l}] = testConvergence(params, sPodSystem, qs) ; 
        sPodSystems{l}             = sPodSystem                              ;
        Sall{l}                    = sPodSystem.S                            ; 
    end
    disp(['cleaning iteration number ' num2str(l)]) 
end

% if checkConvergenceCriteria is true, then orthoError, Sdiff, Sall, and
% sPodSystems are stored
if(checkConvergenceCriteria)
    if(~exist('RESULTS','dir'))
        mkdir('RESULTS')
    end
    save('RESULTS/convergenceCriteria', 'orthoError', 'Sdiff', 'Sall', 'sPodSystems')  ;
end

% relative mean error after last cleaning iteration is computed and
% displayed
relErrors(nCleaningIterations+1) = errorMeasure(qs, qsTilde-qs) ; 
if(l==nCleaningIterations)
    disp(['relative error: ',num2str(relErrors(nCleaningIterations+1))])
end