function [orthoError, Sdiff] = testConvergence(params, sPodSystem, qs) 

% This function computes the convergence criteria introduced in the 2D
% section in the paper.
%
% Output arguments:
% - orthoError: cell array of dimension kx1, where k is the number of
%   frames; each entry contains a vector containing the orthogonality errors
%   for each of the modes of the associated frame
%
% - Sdiff: cell array of dimension kx1, where k is the number of
%   frames; each entry contains a vector containing the first five singular
%   values of the residual
%
%
% Input arguments:
% - params: struct containg parameters which specify the case.
%
% - sPodSystem: struct containing the sPOD modes, their amplitudes, the
%   shift matrices, and the number of frames.
%
% - qs: snapshot tensor of dimension NxQxpxm, where N is the number of grid
%   points in x-direction, Q the number of grid points in y-direction, p
%   the number of physical variables, and m the number of time steps
%
%
% Authors:
%   Julius Reiss, reiss@tnt.tu-berlin.de
%   Philipp Schulze, pschulze@math.tu-berlin.de
%
% Version:
%   1.0 (February 15, 2018)
% 

if(ndims(qs)~=4)
    error('testConvergence is only implemented for 2D examples.')
end

% derived data from params and sPodSystem
nTimeSteps     = params.time.nTimeSteps ; 
Nxi            = params.geometry.Nxi    ;
nModes         = params.mode.nModes     ;
subSpaceNumber =  sPodSystem.nSubSpaces ; 

% computing the residual
qsTilde  = additiveSubspaceCombination(params, sPodSystem) ; % sPOD approximation
diff     = qs-qsTilde                                      ; % residual

% some initializations
Sdiff      = cell(subSpaceNumber, 1)          ; 
diffShifts = zeros([size(qs),subSpaceNumber]) ; % contains shifted residuals
orthoError = cell(subSpaceNumber, 1)          ;  

% computing Sdiff and orthoError
for k=1:subSpaceNumber % loop over the velocity frames
    diffShifts(:,:,:,:,k) = shiftMatrix(params, diff, 1, sPodSystem.shiftMatrices{k}) ;
    Sdiff{k}              = svds(reshape(diffShifts(:,:,:,:,k ),[],nTimeSteps), 5)    ;
    currentModeNumber     = min(nModes(k), size(sPodSystem.S{k},1))                   ; % number of the modes associated to the current velocity frame 
    orthoError{k}         = NaN(currentModeNumber,1)                                  ;
    for l =1:currentModeNumber % loop over the current modes
        qTMode           = reshape(sPodSystem.U{k}(:,l)*sPodSystem.S{k}(l,l)*sPodSystem.V{k}(:,l)', Nxi(1), Nxi(2), 1, nTimeSteps) ; % (reshaped) dyadic product of current mode and its amplitude
        orthoError{k}(l) = ssum(diffShifts(:,:,:,:,k).*qTMode)/sqrt(ssum(diffShifts(:,:,:,:,k).^2)*ssum(qTMode.^2))                ; 
    end
end