% This script performs the mode decomposition based on the shifted proper
% orthogonal decomposition (sPOD).
%
%
% Authors:
%   Julius Reiss, reiss@tnt.tu-berlin.de
%   Philipp Schulze, pschulze@math.tu-berlin.de
%
% Version:
%   1.0 (February 15, 2018)
% 

% derived data from params
nTimeSteps          = params.time.nTimeSteps                                               ;
nDimensions         = params.geometry.nDimensions                                          ;
adaptiveModeAdding  = giveStructValue(params, 'mode', 'adaptiveModeAdding', false)         ;
csPostProcessing    = giveStructValue(params, 'SLdetection', 'postProcessing', @identity)  ;
storeIntermediateCs = giveStructValue(params, 'SLdetection', 'storeIntermediateCs', false) ;

% determing the shift values "cs" based on the specified method 
% params.SLdetection.mode
switch params.SLdetection.mode
    case 'prescribedVelocity' % shifts based on prescribed constant shift velcoities
        t        = params.time.dt*(0:1:nTimeSteps-1) ;
        velocity = params.SLdetection.velocity       ;
        nFrames  = size(velocity,2)                  ; % number of velocity frames
        if(nDimensions==1)
            cs = t'*velocity ;
        else
            cs = NaN(nTimeSteps, nDimensions, nFrames) ;
            for i=1:nFrames
                cs(:,:,i) = (velocity(:,i)*t)' ;
            end
        end
        clear t velocity
    case 'trackedFronts' % shifts are determined via tracking
        if(nDimensions>1)
            error('The option "trackedFronts" for params.SLdetection.mode is only supported if the space dimension is one!')
        end
        nFrames = length(params.SLdetection.trackingFunctions) ; % number of velocity frames
        cs      = NaN(nTimeSteps, nFrames)                     ;
        for i=1:nFrames
            S 	    = load(params.SLdetection.SourceFiles{i})                       ; % loading the snapshot matrix based on which the shifts of the ith velocity frame shall be determined
            cs(:,i) = params.SLdetection.trackingFunctions{i}(S.qs(:,1:nTimeSteps)) ;
        end
        cs = cs*params.geometry.dxi ;   
        clear S
    otherwise
        error('Unknown shift length detection mode.')
end

% if "storeIntermediateCs" is true, then cs is stored before being
% post-processed
if(storeIntermediateCs)
    if(~exist('RESULTS','dir'))
        mkdir('RESULTS')
    end
    save('RESULTS/intermediateCs','cs')
end

% post-processing the shifts
cs = csPostProcessing(cs) ;

% executing the sPOD algorithm
[sPodSystem, qsTilde, relErrors] = subspaceDecompositionBySPODiteration(params, qs, cs ) ;
% sPodSystem is a struct containing sPOD modes, amplitudes, and shift
% matrices
% qsTilde is the sPOD approximation of qs
% relErrors is a vector containing the relative mean error for each
% cleaning iteration

% computing and displaying the minimum relative error
minRelError = min(relErrors) ;
disp(['minimum relative error:' num2str(minRelError)])

% if adaptiveModeAdding is true, then modes are added in a Greedy fashion
% until the error tolerance is met
if(adaptiveModeAdding)
    nSubspacesExtended = nFrames+1 ; % number of velocity frames plus one (respresenting the zero velocity frame)
    if(nDimensions==1)
        csExtended = [cs zeros(nTimeSteps,1)] ; % matrix containing the shift values including the shifts for the zero velocity frame
    else
        error('Adaptive mode adding is only supported for 1D problems.')
    end
    tempRelErrors 	    = NaN(params.mode.nCleaningIterations+1, nSubspacesExtended) ; % initalization of the matrix containing the relative errors for all cleaning iterations and for each velocity frame (one mode is added to each velocity frame at a time)
    tempAddZeroVelocity = false ; % temporary variable stating if the zero velocity frame is added or not
    if(minRelError>=params.mode.relErrorTol) % if error tolerance is already met, the mode adding is not started
        disp(' ')
        disp('Adding modes until tolerance is met')
        clear sPodSystem qsTilde
        while(minRelError>=params.mode.relErrorTol) % continue until tolerance is met
            for i=1:nFrames % add one mode to each of the frames at a time
                params.mode.nModes(i)      = params.mode.nModes(i)+1                ; % mode is added
                [~, ~, tempRelErrors(:,i)] = subspaceDecompositionBySPODiteration(params, qs, cs ) ; % sPOD algorithm is performed and relative errors are computed
                params.mode.nModes(i)      = params.mode.nModes(i)-1                ; % mode is subtracted again
            end
            if(~tempAddZeroVelocity) % if zero velocity frame is not already included, add it temporarily
                params.SLdetection.addZeroVelocity 	   = true                                          ;
                params.mode.nModes(nSubspacesExtended) = 1                                             ; % mode is added to zero velocity frame
                [~, ~, tempRelErrors(:,end)]           = subspaceDecompositionBySPODiteration(params, qs, csExtended) ; % sPOD algorithm is performed and relative errors are computed
                params.mode.nModes(nSubspacesExtended) = 0                                             ; % mode is subtracted again
            end
            minRelError = min(min(tempRelErrors)) ; % the minimum relative error of all cleaning iterations and all frames
            [~, winnerIdx] = find(tempRelErrors==min(min(tempRelErrors)), 1) ; % the index of the frame corresponding to the smallest error
            if(~tempAddZeroVelocity) % if zero velocity frame is not already included
                if(winnerIdx==nSubspacesExtended) % if adding a mode to the zero velocity frame resulted in the smallest error (in comparison to the other frames) 
                    tempAddZeroVelocity = true       ; % add zero velocity frame
                    nFrames             = nFrames+1  ; % accordingly, number of frames needs to be increased ...
                    cs                  = csExtended ; % ... and the matrix containing the shifts has to be extended
                else % if adding a mode to the zero velocity frame did not result in the smallest error
                    params.SLdetection.addZeroVelocity = false                         ; % undo the temporary adding of the zero velocity frame
                    params.mode.nModes                 = params.mode.nModes(1:nFrames) ; % reduce the length of the nModes vector whose length should correspond to the number of frames
                end
            end
            params.mode.nModes(winnerIdx) = params.mode.nModes(winnerIdx)+1 ; % add a mode to the frame corresponding to the smallest error
            disp(['current mode numbers for each frame:' num2str(params.mode.nModes)])
            disp(['minimum relative error:' num2str(minRelError)])
        end
        clear nSubspacesExtended csExtended tempRelErrors tempAddZeroVelocity winnerIdx
        [sPodSystem, qsTilde, relErrors] = subspaceDecompositionBySPODiteration(params, qs, cs ) ; % once again compute the sPOD algorithm with the numbers of modes which are already known to result in an error smaller than the tolerance (necessary since sPodSystem and qsTilde have not yet been computed)
    end
end