function shiftMatrices = shiftMatrixGenerator_constExtrapolation(params, cs)

% This function assembles shift matrices based on constant extrapolation
% over the boundaries and based on a time-dependent shift cs.
%
% Output arguments:
% - shiftMatrices: cell array of dimension m, where m is the length of cs; 
%   each entry is a shift matrix of dimension NxN, where N is the 
%   number of grid points along the spatial direction specified by 
%   params.mode.shiftDirection
%
%
% Input arguments:
% - params: struct containing parameters which specify the case.
%
% - cs: vector containing the time-dependent shift at each discrete time
%   point.
%
%
% Authors:
%   Julius Reiss, reiss@tnt.tu-berlin.de
%   Philipp Schulze, pschulze@math.tu-berlin.de
%
% Version:
%   1.0 (February 15, 2018)
% 

% derived data from params and cs
shiftDirection = giveStructValue(params, 'mode', 'shiftDirection', 1) ;
Nxi            = params.geometry.Nxi(shiftDirection)                  ; % number of grid points along the spatial direction specified by shiftDirection
dxi            = params.geometry.dxi(shiftDirection)                  ; % grid size along the spatial direction specified by shiftDirection
nCs            = length(cs)                                           ; % number of entries in cs

% some initializations
shiftMatrices    = cell(nCs, 1) ; 
diagonalIndexVec = NaN(1, 4)    ; % vector containing the indices of the non-zero diagonals of shiftMatrices

for i=1:nCs
    shift                                  = cs(i)                                        ; % current shift value 
    diagonalIndexVec(2)                    = floor(shift/dxi)                             ;
    diagonalIndexVec(1)                    = diagonalIndexVec(2)-1                        ;
    diagonalIndexVec(3)                    = diagonalIndexVec(2)+1                        ;  
    diagonalIndexVec(4)                    = diagonalIndexVec(2)+2                        ;
    dxiWeight                              = shift/dxi-diagonalIndexVec(2)                ; % weighting factor used for Lagrange interpolation
    Lagrange                               = lagrange(dxiWeight)                          ; % coefficients based on Lagrange interpolation with interpolation stencil [-1 0 1 2]
    entries                                = repmat(Lagrange', Nxi, 1)                    ; % entries of the non-zero diagonals of shiftMatrices{i}
    T                                      = spdiags(entries, diagonalIndexVec, Nxi, Nxi) ; % temporary variable for shiftMatrices{i}
    T(1:min(-diagonalIndexVec(4)+1,Nxi),1) = 1                                            ; % filling up with ones to account for the constant extrapolation
    if((diagonalIndexVec(4)<=1)&&(diagonalIndexVec(4)>=-(Nxi-2)))
        T(-diagonalIndexVec(4)+2,1) = sum(Lagrange(1:3)) ; % adding up coefficients to account for the constant extrapolation
    end
    if((diagonalIndexVec(4)<=2)&&(diagonalIndexVec(4)>=-(Nxi-3)))
        T(-diagonalIndexVec(4)+3,1) = sum(Lagrange(1:2)) ; % adding up coefficients to account for the constant extrapolation
    end
    T(max(end-diagonalIndexVec(1),1):end,end) = 1 ; % filling up with ones to account for the constant extrapolation
    if((diagonalIndexVec(1)>=-1)&&(diagonalIndexVec(1)<=Nxi-2))
        T(end-diagonalIndexVec(1)-1,end) = sum(Lagrange(2:4)) ; % adding up coefficients to account for the constant extrapolation
    end
    if((diagonalIndexVec(1)>=-2)&&(diagonalIndexVec(1)<=Nxi-3))
        T(end-diagonalIndexVec(1)-2,end) = sum(Lagrange(3:4)) ; % adding up coefficients to account for the constant extrapolation
    end
    shiftMatrices{i} = T ; 
end