function shiftMatrices  = shiftMatrixGenerator_periodic(params, velocity)

% This function assembles shift matrices based on periodic boundary
% conditions and based on a constant shift velocity. The assumed 
% periodicity is: 'zero'th value = last value, i.e., boundary is only 
% represented once in the snapshot matrix.
%
% Output arguments:
% - shiftMatrices: cell array of dimension m, where m is the number of time
%   steps; each entry is a shift matrix of dimension NxN, where N is the 
%   number of grid points along the spatial direction specified by 
%   params.mode.shiftDirection
%
%
% Input arguments:
% - params: struct containing parameters which specify the case.
%
% - velocity: scalar; constant shift velocity
%
%
% Authors:
%   Julius Reiss, reiss@tnt.tu-berlin.de
%   Philipp Schulze, pschulze@math.tu-berlin.de
%
% Version:
%   1.0 (February 15, 2018)
% 

% derived data from params
shiftDirection = giveStructValue(params, 'mode', 'shiftDirection', 1)                                ;
Nxi            = params.geometry.Nxi(shiftDirection)                                                 ; % number of grid points along the spatial direction specified by shiftDirection
dxi            = params.geometry.dxi(shiftDirection)                                                 ; % grid size along the spatial direction specified by shiftDirection
t              = giveStructValue(params, 'time', 't', params.time.dt*(0:1:params.time.nTimeSteps-1)) ;
L              = giveStructValue(params, 'geometry', 'L', Nxi*dxi)                                   ;
dt             = t(2:end)-t(1:end-1)                                                                 ; % vector containing the time step sizes

% some initializations
shift            = 0                  ;  % current shift value 
shiftMatrices    = cell(length(t), 1) ; 
diagonalIndexVec = NaN(1, 4)          ; % vector containing the indices of some of the non-zero diagonals of shiftMatrices

for i=1:length(t)
    if(shift>L) % reduce shift if too big to account for the periodicity
        shift = shift-L ; 
    end
    if(shift<0) % increase shift if negative to account for the periodicity
        shift = shift+L ; 
    end
    diagonalIndexVec(2) = floor(shift/dxi)      ;
    diagonalIndexVec(1) = diagonalIndexVec(2)-1 ;
    diagonalIndexVec(3) = diagonalIndexVec(2)+1 ;  
    diagonalIndexVec(4) = diagonalIndexVec(2)+2 ;
    if(diagonalIndexVec(1)<0) % if diagonal index is negative, increase it to account for the periodicity
        diagonalIndexVec(1) = diagonalIndexVec(1)+Nxi ; 
    end
    if(diagonalIndexVec(4)>Nxi) % if diagonal index is too big, increase it to account for the periodicity
        diagonalIndexVec(4) = diagonalIndexVec(4)-Nxi ; 
    end
    dxiWeight            = shift/dxi-diagonalIndexVec(2)                     ; % weighting factor used for Lagrange interpolation
    dualDiagonalIndexVec = diagonalIndexVec-(Nxi)*sign(diagonalIndexVec+0.1) ; % diagonals are continued periodically in the matrix, dualDiagonalIndexVec contains the indices associated to these continued diagonals
    bands                = [diagonalIndexVec dualDiagonalIndexVec]           ; % summarizes the indices of all non-zero diagonals
    Lagrange             = lagrange(dxiWeight)                               ; % coefficients based on Lagrange interpolation with interpolation stencil [-1 0 1 2]
    entries              = repmat(Lagrange', Nxi, 2)                         ; % entries of the non-zero diagonals of shiftMatrices{i}
    shiftMatrices{i}     = spdiags(entries, bands, Nxi, Nxi)                 ;
    if(i<length(t))
        shift = shift+velocity*dt(i) ; % shift is increased based on the constant shift velocity and the time step size
    end
end