function [sPODsystem, params] = initShiftMatrix(params, cs) 

% This function initializes sPODsystem by equipping it with shift matrices
% and the number of velocity frames.
%
% Output arguments:
% - sPODsystem: struct containing shift matrices and the number of frames
%
% - params: struct containing parameters which specify the case.
%
%
% Input arguments:
% - params: struct containing parameters which specify the case.
%
% - cs: matrix of dimension mxk, where m is the number of time steps and k
%   is the number of velocity frames; OR a tensor of dimension mxlxk, where
%   l is the number of spatial dimensions; contains the shift values for
%   each velocity frame and each discrete time step (and for each space 
%   dimension in the tensor case)
%
%
% Authors:
%   Julius Reiss, reiss@tnt.tu-berlin.de
%   Philipp Schulze, pschulze@math.tu-berlin.de
%
% Version:
%   1.0 (February 15, 2018)
% 

% determination of params.mode.shiftDirection which specifies along which
% spatial dimension the shift is performed
if(~ismatrix(cs)) % 2D case
    maximumShiftPerDimension = max(squeeze(max(abs(cs))), [], 2) ;
    if(max(maximumShiftPerDimension)~=sum(maximumShiftPerDimension)) % if shifts are specified in more than one spatial dimension, an error message is displayed since this is not yet supported.
        error('So far, only shifts along one of the spatial dimensions are supported!')
    end
    params.mode.shiftDirection = find(maximumShiftPerDimension==max(maximumShiftPerDimension), 1) ;
    cs                         = squeeze(cs(:,params.mode.shiftDirection,:))                      ; % cs is squeezed from a 3D tensor to a matrix
else % 1D case
    params.mode.shiftDirection = 1 ;
end

% initialization of sPODsystem
sPODsystem               = struct() 		     ;
sPODsystem.nSubSpaces 	 = size(cs,2) 			 ; % number of frames or subspaces
nSubSpaces               = sPODsystem.nSubSpaces ;
sPODsystem.shiftMatrices = cell(nSubSpaces, 1)   ; 

negativeShiftsViaRot90 = giveStructValue(params, 'mode', 'negativeShiftsViaRot90', true) ;

% assembling the shift matrices
for i=1:nSubSpaces
	sPODsystem.shiftMatrices{i} = params.mode.shiftMatrixGenerator(params, cs(:,i)) ;
    if(~negativeShiftsViaRot90) % if negativeShiftsViaRot90 is false, then also the back-shift matrices are computed explicitly
        sPODsystem.negShiftMatrices{i} = params.mode.shiftMatrixGenerator(params, -cs(:,i)) ;
    end
end