function [sPODsystem, params] = initShiftMatrix_constVelocity(params, velocity) 

% This function initializes sPODsystem by equipping it with shift matrices
% and the number of velocity frames.
%
% Output arguments:
% - sPODsystem: struct containing shift matrices and the number of frames
%
% - params: struct containing parameters which specify the case.
%
%
% Input arguments:
% - params: struct containing parameters which specify the case.
%
% - velocity: matrix of dimension lxk, where l is the number of spatial 
%   dimensions and k is the number of velocity frames; contains the 
%   constant shift velocities for each velocity frame and for each space 
%   dimension
%
%
% Authors:
%   Julius Reiss, reiss@tnt.tu-berlin.de
%   Philipp Schulze, pschulze@math.tu-berlin.de
%
% Version:
%   1.0 (February 15, 2018)
% 

% determination of params.mode.shiftDirection which specifies along which
% spatial dimension the shift is performed
if(min(size(velocity))>1)
    maximumShiftVelocityPerDimension = max(abs(velocity), [], 2) ;
    if(max(maximumShiftVelocityPerDimension)~=sum(maximumShiftVelocityPerDimension)) % if shift velocities are specified in more than one spatial dimension, an error message is displayed since this is not yet supported.
        error('So far, only shift velocities along one of the spatial dimensions are supported!')
    end
    params.mode.shiftDirection = find(maximumShiftVelocityPerDimension==max(maximumShiftVelocityPerDimension), 1) ;
    velocity                   = velocity(params.mode.shiftDirection,:)                                           ; % only non-zero row of velocity is kept
else
    params.mode.shiftDirection = 1 ;
end

% initialization of sPODsystem
sPODsystem               = struct() 		     ;
sPODsystem.nSubSpaces 	 = length(velocity) 	 ; % number of frames or subspaces
nSubSpaces               = sPODsystem.nSubSpaces ;
sPODsystem.shiftMatrices = cell(nSubSpaces, 1)   ; 

negativeShiftsViaRot90 = giveStructValue(params, 'mode', 'negativeShiftsViaRot90', true) ;

% assembling the shift matrices
for i=1:nSubSpaces
	sPODsystem.shiftMatrices{i} = params.mode.shiftMatrixGenerator(params, velocity(i)) ;
    if(~negativeShiftsViaRot90) % if negativeShiftsViaRot90 is false, then also the back-shift matrices are computed explicitly
        sPODsystem.negShiftMatrices{i} = params.mode.shiftMatrixGenerator(params, -velocity(i)) ;
    end
end