function qsShifted = shiftMatrix(params, qs, shiftDirection, shiftMatrices)

% This function shifts the matrix or tensor qs based on the shiftMatrices.
%
% Output arguments:
% - qsShifted: matrix or tensor of same dimension as qs; shifted matrix or
%   tensor
%
%
% Input arguments:
% - params: struct containing at least the number of grid points and the
%   number of time steps.
%
% - qs: snapshot tensor
%
% - shiftDirection: 1 or 2; specifies along which spatial dimension the 
%   shift is performed (1 for x-axis and 2 for y-axis)
%
% - shiftMatrices: cell array of dimension m, where m is the number of time
%   steps; each entry is a shift matrix of dimension NxN, where N is the 
%   number of grid points along the spatial direction specified by 
%   shiftDirection
%
%
% Authors:
%   Julius Reiss, reiss@tnt.tu-berlin.de
%   Philipp Schulze, pschulze@math.tu-berlin.de
%
% Version:
%   1.0 (February 15, 2018)
% 

% derived data from params and qs
Nxi        = params.geometry.Nxi    ;
nTimeSteps = params.time.nTimeSteps ;
sizeOfQs   = size(qs)               ;

% in case qs is a vector, shifting is a simple matrix vector multiplication
if(isvector(qs))
    qsShifted = shiftMatrices{1}*qs ; 
    return
end

% if qs is a matrix, reshape it as a 3D tensor
if(numel(sizeOfQs)==2)
    qs = reshape(qs, sizeOfQs(1), 1, sizeOfQs(2)) ;
    sizeOfQs   = size(qs) ;
end

nVars      = sizeOfQs(end-1) ; % number of physical variables

qsShifted  = NaN(sizeOfQs)   ; % initialization of qsShifted

% computing qsShifted
switch numel(sizeOfQs)
    case 3 % 1D case
        for i = 1:nTimeSteps
            for j = 1:nVars
                qsShifted(:,j,i) = shiftMatrices{i}*qs(:,j,i) ; 
            end
        end
    case 4 % 2D case
        switch shiftDirection
            case 1
                for i = 1:nTimeSteps
                    for j = 1:nVars
                        for k=1:Nxi(2)
                            qsShifted(:,k,j,i) = shiftMatrices{i}*qs(:,k,j,i) ; 
                        end
                    end
                end
            case 2
                for i = 1:nTimeSteps
                    for j = 1:nVars
                        for k=1:Nxi(1)
                            qsShifted(k,:,j,i) = shiftMatrices{i}*qs(k,:,j,i) ; 
                        end
                    end
                end
            otherwise
                error('The value of the shiftDirection value should be between 1 and 2.')
        end
    otherwise
        error('The variable qs should have 3 to 4 dimensions!')
end